@@ -749,13 +749,8 @@ thread_chainproc ic command_ok {cmd} {
 	    }
 	    sauceadmin {
 		if {![string length $rhs]} {
-		    set chal [exec -keepnewline \
-			    dd if=/dev/urandom bs=1 count=8 2>/dev/null]
-		    binary scan $chal H* chal
-		    if {[string length $chal] != 16} {
-			error "urandom failed `$chal'"
-		    }
-		    append chal [format %08lx [clock seconds]]
+		    set chal [urandom_hex 8]
+		    append chal [format %016lx [clock seconds]]
 		    set state(lastchal) $chal
 		    ic_commandresponse immed "393 $chal"
 		} elseif {![info exists state(lastchal)]} {
@@ -765,18 +760,18 @@ thread_chainproc ic command_ok {cmd} {
 		    set waschal $state(lastchal)
 		    log notice "$state(desc): ATTEMPTING SWITCH TO ADMIN MODE"
 		    if {![regexp \
-           {^([0-9a-f]{16})([0-9a-f]{8})[ \t]+([0-9a-f]{32})$} \
+           {^([0-9a-f]{16})([0-9a-f]{16})[ \t]+([0-9a-f]{64})$} \
 			    $rhs all chal wasdate resp]} {
 			ic_commandresponse major "501 \\x{24} \\x{32} please"
 		    } elseif {"$chal$wasdate" != "$waschal"} {
 			ic_commandresponse immed "490 challenge overwritten"
-		    } elseif "[clock seconds] - 0x$wasdate \
-			    > $admin_chal_timeout" {
+		    } elseif {[clock seconds] - [expr "0x$wasdate"] \
+			    > $admin_chal_timeout} {
 			ic_commandresponse immed "491 challenge timed out"
 		    } elseif {![string length $adminsecret]} {
 			ic_commandresponse immed "495 admin secret missing"
 		    } elseif {"$resp" != \
- "[exec <<"[binary format H* $waschal]$adminsecret" md5sum]"} {
+ "[hbcrypto hash sha256 [hbytes concat $waschal $adminsecret]]"} {
                         ic_commandresponse immed "492 incorrect response"
                         unset state(lastchal)
                     } else {
